from .chat import Chat

MALE = 1
FEMALE = 2


class User(Chat):
    """
    好友(:class:`Friend`)、群聊成员(:class:`Member`)，和公众号(:class:`MP`) 的基础类
    """

    def __init__(self, raw, bot):
        super(User, self).__init__(raw, bot)

    @property
    def remark_name(self):
        """
        备注名称
        """
        return self.raw.get('RemarkName')

    @property
    def sex(self):
        """
        性别
        """
        return self.raw.get('Sex')

    @property
    def province(self):
        """
        省份
        """
        return self.raw.get('Province')

    @property
    def city(self):
        """
        城市
        """
        return self.raw.get('City')

    @property
    def signature(self):
        """
        个性签名
        """
        return self.raw.get('Signature')

    def add(self, verify_content=''):
        """
        把当前用户加为好友

        :param verify_content: 验证信息(文本)
        """
        return self.bot.add_friend(user=self, verify_content=verify_content)

    def accept(self, verify_content=''):
        """
        接受当前用户为好友

        :param verify_content: 验证信息(文本)
        :return: 新的好友对象
        """
        return self.bot.accept_friend(user=self, verify_content=verify_content)

    @property
    def is_friend(self):
        """
        判断当前用户是否为好友关系

        :return: 若为好友关系则为 True，否则为 False
        """
        if self.bot:
            return self in self.bot.friends()



from time import gmtime, strftime
from docx.shared import Inches
from docx.shared import Pt
import matplotlib.pyplot as plt
import seaborn as sns
import seaborn.linearmodels as snsl
import tushare as ts
from docx import Document
from docx.enum.style import WD_STYLE_TYPE
from timeit import default_timer as timer
from datetime import datetime, timedelta
import matplotlib.dates as mdates
from matplotlib.pyplot import subplots, draw
from matplotlib.finance import _candlestick
import pandas as pd
from pandas.tseries.offsets import BDay

def shiftFromToday(day, isBack): #int
    today = pd.datetime.today()
    if isBack:
        theday = today - BDay(day)
        print(theday)
        theday = '{}'.format(theday)
        theday = theday.split(' ')[0]
        return theday
    else:
        theday = today + BDay(day)
        print(theday)
        theday = '{}'.format(theday)
        theday = theday.split(' ')[0]
        return theday
def today():
    return '{}'.format(strftime("%Y-%m-%d", gmtime()))  # "%Y-%m-%d %H:%M:%S"
def yesterday():
    return '{}'.format(datetime.strftime(datetime.now() - timedelta(1), '%Y-%m-%d'))


def templeateForming(): #with layout
    # TEMPLATE FORMING
    template = "tushareUse/Files/input.docx"
    document = ts.MailMerge(template)
    document.merge(
        today=today(),
        p1='Springfield',
        p2='800-555-5555')
    document.write('tushareUse/Files/output.docx')

#Document editing
wordDocument = Document('tushareUse/Files/output.docx')

def newDocument():
    wordDocument._body.clear_content()

def openDocument():
    p = wordDocument.add_paragraph()
    setParagraphStyle(p)
    r = p.add_run()
    return r

def saveDocument():
    wordDocument.save('tushareUse/Files/output.docx')

def addTableToDocument(rows, cols):
    # styles = [s for s in wordDocument.styles if s.type == WD_STYLE_TYPE.TABLE]
    # for style in styles:
    #     print(style.name)
    table = wordDocument.add_table(rows = rows, cols = cols)
    table.style = 'Light Shading Accent 3'
    return table

def setParagraphStyle(p):
    style = wordDocument.styles['Normal']
    font = style.font
    font.name = 'Arial'
    font.size = Pt(15)
    p.style = wordDocument.styles['Normal']

def setPageMargin(inputfile): ## not tested
    # Open the document
    document = Document(outputfile)
    # changing the page margins
    sections = document.sections
    for section in sections:
        section.top_margin = Cm(0)
        section.bottom_margin = Cm(0)
        section.left_margin = Cm(0)
        section.right_margin = Cm(0)

    document.save(args.outputFile)

def drawCandlestick(df):
    #weekFormatter = DateFormatter('%b %d')     # 如：Jan 12
    fig, ax = plt.subplots()
    fig.subplots_adjust(bottom=0.2)
    df['t'] = df['date']
    _candlestick(ax, df, width=0.6, colorup='r', colordown='g')

    ax.xaxis_date()
    ax.autoscale_view()
    plt.setp(plt.gca().get_xticklabels(), rotation=45, horizontalalignment='right')


    ax.grid(True)
    plt.title('000777', fontproperties=zhfont)
    plt.show()

def getCodeArray():
    allStocks = ts.get_today_all()
    codeArray = []
    for i in range(0, allStocks.shape[0]):  # pandas dataframe row count, column count is stock.shape[1]
        codeArray.append(allStocks.iloc[i]['code'])
    return codeArray


from .console import embed, shell_entry
from .misc import ensure_list, get_receiver, get_user_name, handle_response, smart_map, match_attributes, \
    match_name, match_text, wrap_user_name
from .tools import dont_raise_response_error, ensure_one, mutual_friends



from .Helper import *

#save images to word
def saveImagesToWord(title,content,numberOfPictures):
    r = openDocument()
    r.add_text(title)
    r.add_text(content)
    no = 0
    for i in range(0, numberOfPictures):
        no = no + 1
        filename = 'Files/Images/i{}.png'.format(no)
        r.add_picture(filename)
    saveDocument()

#save text to word
def saveTextToWord(content):
    r = openDocument()
    r.add_text(content)
    saveDocument()

#save table to word
def saveTableToWord(name, content):
    r = openDocument()
    r.add_text('                                                                                                                                                                           ')
    r.add_text('\nTable Name: {}'.format(name))
    if name == 'basics':
        table = addTableToDocument(content.shape[0],2)
        for i in range(0, content.shape[0]):
            table.rows[i].cells[0].text = ['{}'.format(content.iloc[[i]])][0].split('\n')[0].split('   ')[0]
            table.rows[i].cells[1].text = ['{}'.format(content.iloc[[i]])][0].split('\n')[0].split('   ')[1]
    else:
        table = addTableToDocument(content.shape[0]+1, content.shape[1])
        for j in range(content.shape[-1]):
            table.cell(0,j).text = content.columns[j]
            # if j>12: #will experience serious lag
            #     break
        for i in range(content.shape[0]):
            # if i > 12:
            #     break
            for j in range(content.shape[-1]):
                # if j >12:
                #     break
                table.cell(i+1,j).text = str(content.values[i,j])
    saveDocument()

#save images to local and close
def saveImage(plt,noOfImage):
    filename = 'Files/Images/i{}.png'.format(noOfImage)
    plt.savefig(filename)
    plt.close()#comment this if need to see images


#!/usr/bin/env python3
# coding: utf-8
from .Start import *

__title__ = 'BrosHelp'
__version__ = '0.0.1'
__author__ = 'Kay'
__license__ = 'MIT'
__copyright__ = '2017, Kay'
version_details = 'BrosHelp {ver} from {path} (python {pv.major}.{pv.minor}.{pv.micro})'.format(
    ver=__version__, path=__path__[0], pv=sys.version_info)



import logging

from wxpy.utils import get_receiver

logger = logging.getLogger(__name__)


class WeChatLoggingHandler(logging.Handler):
    def __init__(self, receiver=None):
        """
        可向指定微信聊天对象发送日志的 Logging Handler

        :param receiver:
            * 当为 `None`, `True` 或字符串时，将以该值作为 `cache_path` 参数启动一个新的机器人，并发送到该机器人的"文件传输助手"
            * 当为 :class:`机器人 <Bot>` 时，将发送到该机器人的"文件传输助手"
            * 当为 :class:`聊天对象 <Chat>` 时，将发送到该聊天对象
        """

        super(WeChatLoggingHandler, self).__init__()
        self.receiver = get_receiver(receiver)

    def emit(self, record):
        # noinspection PyBroadException
        try:
            self.receiver.send(self.format(record))
        except:
            # Todo: 将异常输出到屏幕
            pass


def get_wechat_logger(receiver=None, name=None, level=logging.WARNING):
    """
    获得一个可向指定微信聊天对象发送日志的 Logger

    :param receiver:
        * 当为 `None`, `True` 或字符串时，将以该值作为 `cache_path` 参数启动一个新的机器人，并发送到该机器人的"文件传输助手"
        * 当为 :class:`机器人 <Bot>` 时，将发送到该机器人的"文件传输助手"
        * 当为 :class:`聊天对象 <Chat>` 时，将发送到该聊天对象
    :param name: Logger 名称
    :param level: Logger 等级，默认为 `logging.WARNING`
    :return: Logger
    """

    _logger = logging.getLogger(name=name)
    _logger.setLevel(level=level)
    _logger.addHandler(WeChatLoggingHandler(receiver=receiver))

    return _logger



print("STARTED .....")
from .addValue import *
from .Helper import *
from .Save import *
from tabulate import tabulate




def movie():
    movie = ts.realtime_boxoffice()
    movieDetail = ''
    for i in range(0, movie.shape[0]):
        rank = movie.iloc[i]['Irank']
        name = movie.iloc[i]['MovieName']
        sumBoxOffice = movie.iloc[i]['sumBoxOffice']
        boxPerMovie = movie.iloc[i]['boxPer']
        movieDay = movie.iloc[i]['movieDay']
        movieD = '['+ rank + ' '+ name + ' ' + 'day' + movieDay + '] '
        # movieD = rank + ' '+ name + ' ' + movieDay + '天'+ ' 总票房' + sumBoxOffice + ' 票房/影院' + boxPerMovie + '|'
        movieDetail = movieDetail+ movieD
    return movieDetail

'''Adding Columns to stock dataframe'''
# stock = addValue(stock, 'Daily Return')
# stock = addValue(stock, 'MACD')
# stock = addValue(stock,'Double-volume')

''' MATPLOT PLOT and SHOW and SAVE'''
# stock['close'].plot(legend=True ,figsize=(10,4))
# stock['Daily Return'] = stock['close'].pct_change()
# stock['Daily Return'].plot(legend=True,figsize=(10,4))
# stock['volume'].plot(legend=True,figsize=(8,4))
# stock[['MACD','9-day-signal-MACD']].plot(legend=True,figsize=(8,4))
# stock['Double-volume'].plot(legend=True,figsize=(8,4))
# saveImage(plt,1)

''' SAVE to WORD'''
# saveImagesToWord('Double-Volume',1)

''' SEABORN STYLE'''
# sns.set_style("whitegrid")

''' SEABORN  Plot & SAVE'''
# sns.kdeplot(stock['Daily Return'].dropna())
# sns.jointplot(stock['Daily Return'],stock['Daily Return'],alpha=0.2)
# sns_plot = sns.kdeplot(stock['Daily Return'])
# fig = sns_plot.get_figure()
# fig.savefig('Files/output.png')

''' PLOT SHOW '''
# plt.show()

''' TEST '''
# print(stock)
# print(movie)






import logging

from wxpy.utils import ensure_list

logger = logging.getLogger(__name__)


class MessageConfig(object):
    """
    单个消息注册配置
    """

    def __init__(
            self, bot, func, senders, msg_types,
            except_self, run_async, enabled
    ):
        self.bot = bot
        self.func = func

        self.senders = ensure_list(senders)
        self.msg_types = ensure_list(msg_types)
        self.except_self = except_self
        self.run_async = run_async

        self._enabled = None
        self.enabled = enabled

    @property
    def enabled(self):
        """
        配置的开启状态
        """
        return self._enabled

    @enabled.setter
    def enabled(self, boolean):
        """
        设置配置的开启状态
        """
        self._enabled = boolean
        logger.info(self.__repr__())

    def __repr__(self):
        return '<{}: {}: {} ({}{})>'.format(
            self.__class__.__name__,
            self.bot.self.name,
            self.func.__name__,
            'Async, ' if self.run_async else '',
            'Enabled' if self.enabled else 'Disabled',
        )



from wxpy.utils import match_attributes, match_text


class Messages(list):
    """
    多条消息的合集，可用于记录或搜索
    """

    def __init__(self, msg_list=None, bot=None, max_history=None):
        if msg_list:
            super(Messages, self).__init__(msg_list)
        self.bot = bot
        self.max_history = max_history

    def append(self, msg):
        if isinstance(self.max_history, int) and self.max_history > 0:
            del self[:-self.max_history + 1]
        return super(Messages, self).append(msg)

    def search(self, keywords=None, **attributes):
        """
        搜索消息记录

        :param keywords: 文本关键词
        :param attributes: 属性键值对
        :return: 所有匹配的消息
        """

        def match(msg):
            if not match_text(msg.text, keywords):
                return
            if not match_attributes(msg, **attributes):
                return
            return True

        return Messages(filter(match, self), bot=self.bot, max_history=self.max_history)



import pandas as pd

def addValue(stock,value):
    if value == 'Daily Return':
        dailyReturn = []
        for i in range(0, stock.shape[0]):  # pandas dataframe row count, column count is stock.shape[1]
            dailyR = (stock.iloc[i]['price_change'] / stock.iloc[i]['open'])
            dailyReturn.append(dailyR)
        dailyReturnSeries = pd.Series(dailyReturn)  # pandas dataframe add column
        stock['Daily Return'] = dailyReturnSeries.values

    if value == 'MACD':
        MACD = []
        Signal9Day = []
        EMA12 = []
        EMA26 = []
        EMA12Previous = 0
        EMA26Previous = 0
        signal9DayPrevious = 0
        for i in range(0, stock.shape[0]):
            closeP = stock.iloc[i]['close']
            EMA12Previous = closeP*2/(12+1)+ EMA12Previous*(1-2/(12+1))
            oneEMA12 = EMA12Previous
            EMA12.append(oneEMA12)
            EMA26Previous = closeP*2/(26+1)+ EMA26Previous*(1-2/(26+1))
            oneEMA26 = EMA26Previous
            EMA26.append(oneEMA26)
            oneMACD = oneEMA12 - oneEMA26
            MACD.append(oneMACD)
            signal9DayPrevious = oneMACD*2/(9+1) + signal9DayPrevious*(1-2/(9+1))
            oneSignal9Day = signal9DayPrevious
            Signal9Day.append(oneSignal9Day)
        stock['MACD'] = pd.Series(MACD).values# pandas dataframe add column
        stock['9-day-signal-MACD'] = pd.Series(Signal9Day).values# pandas dataframe add column

    if value == 'Double-volume':
        doubleVolume = []
        volumePrevious = 0
        for i in range(0, stock.shape[0]):
            volume = stock.iloc[i]['volume']
            if volumePrevious == 0:
                volumePrevious = volume
            if (volume/volumePrevious)>=2:
                doubleVolume.append(volume)
            else:
                doubleVolume.append(0)
            volumePrevious = volume
        stock['Double-volume'] = pd.Series(doubleVolume).values




    return stock





#!/usr/bin/env python3
# coding: utf-8

from wxpy import *
from tushareUse.tushareMain import movie
import re
# movie()
#    variables   #
when = 'no time'
what = 'nothing'
who = 'somebody?'
eventDate = '2017.3.6'
dateNow = '2017.3.5'
todayOrTommrrow = 'today'
roughData = '没活动'
tulingSwitch = False

#####  bots  #####
bot = Bot()
tulingBot = Tuling(api_key='dd9c07a83af145c3aa72fe3261959e23')

#   setup for person and @ingroups #
allFriends = bot.friends()

@bot.register(allFriends, except_self=False)  # except_self default is True
def auto_reply(msg):
    global roughData
    global when
    global what
    global who
    global tulingSwitch
    if not (isinstance(msg.sender, Group) and not msg.is_at):  # don't reply group unless @
        print('{}'.format(msg.receiver))
        message = msg.text
        senderName = msg.sender.remark_name
        print('testit 1')
        if len(senderName) < 1:
            receiverFull = '{}'.format(msg.receiver)
            print(receiverFull)
            if not len(receiverFull) < 1:
                self = receiverFull.partition(' ')[2]
                self = self[:-1]
                senderName = self
            else:
                senderName = 'botOwner'
        if senderName == '约吧':
            senderName = 'botOwner'
        print('{}:'.format(senderName))
        print(message)
        if message == '约':  # 查看
            print('testit 2')
            if when == 'no time':
                return '［还没人创建活动!  输入“约 时间 活动”譬如：“约 7am Overwatch”来创建活动］'
            else:
                return '［{}点,有{}一起干{}  //输入"约 {}"来参加活动］'.format(when, who, what, when)  # //输入 “约 时间” e.g. "约 7a"来参加活动
        elif (message.partition(' ')[0] == '约') and (len(msg.text) > 2):
            print('testit 3')
            roughData = message.partition(' ')[2]
            tokens = re.findall('\s+', roughData)
            spaceNo = 0
            for i in range(0, len(tokens)):
                spaceNo = spaceNo + len(tokens[i])
            if spaceNo == 1:  # 建立
                when = message.split(' ')[1]
                what = message.split(' ')[2]
                who = senderName
                #                print('{}'.format(chat.user_name))
                #                bot.create_group(users = chat.user_name)
                return '［好的，记下了, 约在: {}点, 干{}］'.format(when, what)
            elif spaceNo == 0:  # 参加
                if message.split(' ')[1] == when:
                    who = '{}&{}'.format(who, senderName)
                    print('{}'.format(senderName))
                    chat = bot.chats().search(senderName)[0]
                    return '［你参加了{}点的{}，有{}］'.format(when, what, who)
                else:
                    return '［这个点儿没活动］'
            else:
                return '［格式不对呀，sb, 你这样写：邀约："约 7a Overwatch“ 参加：”约 7a“ 查看：“约”］'
        elif message == 'Tulingswitch':
            if tulingSwitch == True:
                tulingSwitch = False
                return '<机器人关闭>'
            else:
                tulingSwitch = True
                return '<机器人开启>'
        elif message == 'movie':
            return movie()
        else:
            print('random message, reply with Tuling')
            if tulingSwitch == True:
                return '[{}]'.format(tulingBot.reply_text(msg))
            else:
                return None
            #            return '[{}]'.format(tulingBot.reply_text(msg))
            #            return None
    else:
        print('message from group and I don\'t want to reply')
        return None


print("来约吧((((；ﾟДﾟ)))))))")


@bot.register(msg_types=CARD, except_self=False)
def reply_text(msg):
    print(msg.raw)
    msg.chat.send_raw_msg(msg.raw['MsgType'], msg.raw['Content'])

@bot.register(msg_types=FRIENDS)
# 自动接受验证信息中包含 'wxpy' 的好友请求
def auto_accept_friends(msg):
    # 判断好友请求中的验证文本
    new_friend = bot.accept_friend(msg.card)
    # new_friend.send('Hello, please check my Moments to see how does BroHelp work!')
    new_friend.send_image('pic.png')
    new_friend.send('https://goo.gl/forms/T4bv09npj5XEz9Es1')




bot.start()




import inspect


def _ipython(local, banner):
    from IPython.terminal.embed import InteractiveShellEmbed
    from IPython.terminal.ipapp import load_default_config

    InteractiveShellEmbed.clear_instance()
    shell = InteractiveShellEmbed.instance(
        banner1=banner,
        user_ns=local,
        config=load_default_config()
    )
    shell()


def _bpython(local, banner):
    import bpython

    bpython.embed(locals_=local, banner=banner)


def _python(local, banner):
    import code

    try:
        # noinspection PyUnresolvedReferences
        import readline
    except ImportError:
        pass
    else:
        import rlcompleter
        readline.parse_and_bind('tab:complete')

    code.interact(local=local, banner=banner)


def embed(local=None, banner='', shell=None):
    """
    | 进入交互式的 Python 命令行界面，并堵塞当前线程。
    | 支持使用 bpython，ipython，以及原生 python。

    :param str shell:
        | 指定命令行类型，可设为 'ipython'，'bpython'，'python'，或它们的首字母；
        | 若为 `None`，则按上述优先级进入首个可用的 Python 命令行。
    :param dict local: 设定本地变量环境，若为 `None`，则获取进入之前的变量环境。
    :param str banner: 设定欢迎内容，将在进入命令行后展示。
    """

    import inspect

    if local is None:
        local = inspect.currentframe().f_back.f_locals

    if isinstance(shell, str):
        shell = shell.strip().lower()
        if shell.startswith('b'):
            shell = _bpython
        elif shell.startswith('i'):
            shell = _ipython
        elif shell.startswith('p') or not shell:
            shell = _python

    for _shell in shell, _ipython, _bpython, _python:
        try:
            _shell(local=local, banner=banner)
        except (TypeError, ImportError):
            continue
        except KeyboardInterrupt:
            break
        else:
            break


def get_args():
    import argparse

    ap = argparse.ArgumentParser(
        description='Run a wxpy-ready python console.')

    ap.add_argument(
        'bot', type=str, nargs='*',
        help='One or more variable name(s) for bot(s) to init (default: None).')

    ap.add_argument(
        '-c', '--cache', action='store_true',
        help='Cache session(s) for a short time, or load session(s) from cache '
             '(default: disabled).')

    ap.add_argument(
        '-q', '--console_qr', type=int, default=False, metavar='width',
        help='The width for console_qr (default: None).')

    ap.add_argument(
        '-l', '--logging_level', type=str, default='INFO', metavar='level',
        help='Logging level (default: INFO).')

    ap.add_argument(
        '-s', '--shell', type=str, default=None, metavar='shell',
        help='Specify which shell to use: ipython, bpython, or python '
             '(default: the first available).')

    ap.add_argument(
        '-v', '--version', action='store_true',
        help='Show version and exit.')

    return ap.parse_args()


def shell_entry():
    import re

    import logging
    import wxpy

    args = get_args()

    if args.version:
        print(wxpy.version_details)
        return

    def get_logging_level():
        logging_level = args.logging_level.upper()
        for level in 'CRITICAL', 'ERROR', 'WARNING', 'INFO', 'DEBUG', 'NOTSET':
            if level.startswith(logging_level):
                return getattr(logging, level)
        else:
            return logging.INFO

    logging.getLogger('wxpy').setLevel(get_logging_level())

    try:
        bots = dict()
        for name in args.bot:
            if not re.match(r'\w+$', name):
                continue
            cache_path = 'wxpy_{}.pkl'.format(name) if args.cache else None
            bots[name] = wxpy.Bot(cache_path=cache_path, console_qr=args.console_qr)
    except KeyboardInterrupt:
        return

    banner = 'from wxpy import *\n'

    for k, v in bots.items():
        banner += '{}: {}\n'.format(k, v)

    module_members = dict(inspect.getmembers(wxpy))

    embed(
        local=dict(module_members, **bots),
        banner=banner,
        shell=args.shell
    )



#python3 setup.py register
#python3 setup.py sdist bdist_wheel --universal upload
# use above in same directory of setup.py to upload to PYPI
from setuptools import setup, find_packages

setup(
    name='GoGenji',
    version='0.1.0',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    platforms = 'any',
    include_package_data = True,
    description='GoGenji is a self helper, connecting the whole world within Wechat in peace',
    url='https://github.com/KayShenA/PythonProjects',
    author='Kay',
    author_email='skyluckyrrb@126.com',
    license='MIT',
    entry_points={
        'console_scripts': [
            'GoGenji = GoGenji:start'
        ]
    },
    install_requires=[
        'itchat==1.2.32',
        'requests',
        # 'wxpy',
        ##below for tushares
        'seaborn',
        'matplotlib',
        'python-docx',
        'tushare',
        'pandas',
    ],)



import logging

from .chat import Chat
from .chats import Chats
from .member import Member
from wxpy.utils import ensure_list, get_user_name, handle_response, wrap_user_name

logger = logging.getLogger(__name__)


class Group(Chat):
    """
    群聊对象
    """

    def __init__(self, raw, bot):
        super(Group, self).__init__(raw, bot)

    @property
    def members(self):
        """
        群聊的成员列表
        """
        def raw_member_list(update=False):
            if update:
                self.update_group()
            return self.raw.get('MemberList', list())

        ret = Chats(source=self)
        for raw in raw_member_list() or raw_member_list(True):
            ret.append(Member(raw, self))
        return ret

    def __contains__(self, user):
        user_name = get_user_name(user)
        for member in self.members:
            if member.user_name == user_name:
                return member

    def __iter__(self):
        for member in self.members:
            yield member

    def __len__(self):
        return len(self.members)

    def search(self, name=None, **attributes):
        """
        在群聊中搜索成员

        :param name: 成员名称关键词
        :param attributes: 属性键值对
        :return: 匹配的群聊成员
        """
        return self.members.search(name, **attributes)

    @property
    def owner(self):
        """
        返回群主对象
        """
        owner_user_name = self.raw.get('ChatRoomOwner')
        if owner_user_name:
            for member in self:
                if member.user_name == owner_user_name:
                    return member
        elif self.members:
            return self.members[0]

    @property
    def is_owner(self):
        """
        判断所属 bot 是否为群管理员
        """
        return self.raw.get('IsOwner') == 1 or self.owner == self.bot.self

    def update_group(self, members_details=False):
        """
        更新群聊的信息

        :param members_details: 是否包括群聊成员的详细信息 (地区、性别、签名等)
        """

        @handle_response()
        def do():
            return self.bot.core.update_chatroom(self.user_name, members_details)

        super(Group, self).__init__(do(), self.bot)

    @handle_response()
    def add_members(self, users, use_invitation=False):
        """
        向群聊中加入用户

        :param users: 待加入的用户列表或单个用户
        :param use_invitation: 使用发送邀请的方式
        """

        return self.bot.core.add_member_into_chatroom(
            self.user_name,
            ensure_list(wrap_user_name(users)),
            use_invitation
        )

    @handle_response()
    def remove_members(self, members):
        """
        从群聊中移除用户

        :param members: 待移除的用户列表或单个用户
        """

        return self.bot.core.delete_member_from_chatroom(
            self.user_name,
            ensure_list(wrap_user_name(members))
        )

    def rename_group(self, name):
        """
        修改群聊名称

        :param name: 新的名称，超长部分会被截断 (最长32字节)
        """

        encodings = ('gbk', 'utf-8')

        trimmed = False

        for ecd in encodings:
            for length in range(32, 24, -1):
                try:
                    name = bytes(name.encode(ecd))[:length].decode(ecd)
                except (UnicodeEncodeError, UnicodeDecodeError):
                    continue
                else:
                    trimmed = True
                    break
            if trimmed:
                break

        @handle_response()
        def do():
            if self.name != name:
                logger.info('renaming group: {} => {}'.format(self.name, name))
                return self.bot.core.set_chatroom_name(get_user_name(self), name)

        ret = do()
        self.update_group()
        return ret
